@@ -15,6 +15,7 @@ use Mozilla::CA;
 use Scalar::Util;
 use XML::LibXML;
 
+our $VERSION = '1.0.1';
 
 use constant ENDPOINTS => {
     'IMGUR'           => 'https://api.imgur.com/3',
@@ -46,31 +47,32 @@ sub _ua { shift->{'user_agent'} }
 sub request {
     my ($self, $uri, $http_method, $data, $hdr) = @_;
 
-    $http_method = $http_method ? uc $http_method : 'GET';
+    # Ensure HTTP method is uppercase
+    $http_method = uc($http_method // 'GET');
 
-    my $endpoint = (defined $self->{'rapidapi_key'} ? ENDPOINTS->{'RAPIDAPI'} . $uri : ENDPOINTS->{'IMGUR'} . $uri);
+    # Determine endpoint based on API key presence
+    my $endpoint = $self->{'rapidapi_key'} ? ENDPOINTS->{'RAPIDAPI'} . $uri : ENDPOINTS->{'IMGUR'} . $uri;
 
-    $endpoint  = ($uri =~ /^http(?:s)?/ ? $uri : $endpoint);
+    # Append format type and method to the endpoint
     $endpoint .= ($endpoint =~ /\?/ ? '&' : '?') . '_format=' . $self->{'format_type'} . "&_method=$http_method";
 
-    $self->_ua->default_header('User-Agent' => "ImgurAPI-perl-lib/0.1");
+    # Set User-Agent header
+    $self->_ua->default_header('User-Agent' => "ImgurAPI::Client/$VERSION");
 
+    # Set Authorization header based on authentication type
     if ($self->{'auth'}) {
-        if (my $access_token = $self->{'access_token'}) {
-            $self->_ua->default_header('Authorization' => "Bearer $access_token");
-        } else {
-            die "missing required access_token";
-        }
+        my $access_token = $self->{'access_token'} // die "Missing required access_token";
+        $self->_ua->default_header('Authorization' => "Bearer $access_token");
     } elsif ($self->{'client_id'}) {
         $self->_ua->default_header('Authorization' => "Client-ID " . $self->{'client_id'});
     }
 
-    if ($http_method =~ /^GET|DELETE$/ && scalar keys %$data) {
-        while (my ($key, $value) = each %$data) {
-            $endpoint .= "&$key=$value";
-        }
+    # Append data to the endpoint for GET and DELETE requests
+    if ($http_method =~ /^GET|DELETE$/ && $data && ref($data) eq 'HASH') {
+        $endpoint .= "&$_=$data->{$_}" foreach keys %$data;
     }
 
+    # Create HTTP request object
     my $request;
     if ($http_method eq 'POST') {
         $request = HTTP::Request::Common::POST($endpoint, %{$hdr//{}}, Content => $data);
@@ -80,28 +82,32 @@ sub request {
         $request = HTTP::Request->new($http_method, $endpoint);
     }
 
-    my $resp = $self->_ua->request($request);
-    my @ratelimit_headers = qw(userlimit userremaining userreset clientlimit clientremaining);
+    # Send the request
+    my $response = $self->_ua->request($request);
 
+    # Extract rate limit headers
+    my @ratelimit_headers = qw(userlimit userremaining userreset clientlimit clientremaining);
     foreach my $header (@ratelimit_headers) {
-        $self->{'ratelimit_hdrs'}->{$header} = $resp->header("x-ratelimit-$header");
+        $self->{'ratelimit_hdrs'}->{$header} = $response->header("x-ratelimit-$header");
     }
 
-    $self->{'response'} = $resp;
-    $self->{'response_content'} = $resp->{'_content'};
+    # Store response content
+    $self->{'response'} = $response;
+    $self->{'response_content'} = $response->decoded_content;
 
+    # Print response content for debugging
     print Dumper $self->{'response_content'} if $ENV{'DEBUG'};
 
+    # Decode response based on format type
     if ($self->format_type eq 'xml') {
-        return (XML::LibXML->new)->load_xml(string => $resp->{'_content'});
+        return XML::LibXML->new->load_xml(string => $response->decoded_content);
+    } else {
+        my $decoded = eval { decode_json $response->decoded_content };
+        if (my $err = $@) {
+            die "Failed to decode JSON response: $err\n" . $response->decoded_content . "\n";
+        }
+        return $decoded;
     }
-
-    my $decoded = eval { decode_json $resp->{'_content'} };
-    if (my $err = $@) {
-        die "failed to decode json response: $err\n" . $resp->{'_content'} . "\n";
-    }
-
-    return $decoded;
 }
 
 # Setters
@@ -544,7 +550,6 @@ sub comment_reply {
     return $self->request("/comment/$comment_id", 'POST', $data);
 }
 
-# TODO: test this
 sub comment_report {
     my $self = shift;
     my $comment_id = shift or die "missing required comment_id";
@@ -840,17 +845,52 @@ ImgurAPI::Client - Imgur API client
 This is a client module for interfacing with the Imgur API.
 
 =head1 SYNOPSIS
+=head2 Instantiation
 
-    use ImgurAPI;
+    use ImgurAPI::Client;
 
     my $client = ImgurAPI::Client->new({
-        'client_id'     => 'your_client_id',
-        'client_secret' => 'your_client',
-        'access_token'  => 'your_access_token'
+        'client_id'    => 'your_client_id',
+        'access_token' => 'your_access_token'
     });
 
     my $upload = $client->image_upload("helloimgur.png", 'file', {title => 'title', description => 'desc'});
-    my $image_info = $client->image($upload->{'data'}->{'id'});
+    my $image_info = $client->image($upload->{'data'}->{'id'};
+
+=head2 Authorization
+
+Imgur uses OAuth 2.0 for authentication. OAuth 2.0 has four steps: registration, authorization, making authenticated requests and getting new access tokens after the initial one expires using a refresh token and client secret.
+
+After registering a client application with Imgur L<here|https://api.imgur.com/oauth2/addclient>, the user will need to manually authorize it. Generate a authorization url using the C<oauth2_authorize_url> method and redirect the user to the generated url. The user will be prompted to authorize the application and upon authorization, the user will be redirected to the callback endpoint URL that was specified during application registration. The callback endpoint should collect the access token and refresh token and store them somewhere your code on the backend can pull the access token from and then pass it to the client. You can also visit the authorization url in the browser and manually pull the access token, refresh token and other parameters out of the redirect url and store them somewhere your code can pull them without having a collector endpoint setup. View the official imgur documentation for authorization L<here|https://apidocs.imgur.com/#authorization-and-oauth>.
+
+=head2 Authentication
+
+The client can be authenticated by setting the access token and client id. The access token can be set using the C<set_access_token> method and the client id can be set using the C<set_client_id> method, or by passing them in constructor args.
+
+    my $client = ImgurAPI::Client->new;
+    $client->set_access_token('your_access_token');
+    $client->set_client_id('your_client_id');
+    # OR
+    my $client = ImgurAPI::Client->new({
+        'client_id'    => 'your_client_id',
+        'access_token' => 'your_access_token'
+    });
+
+=head2 METHODS
+=head3 new
+
+    $client = ImgurAPI::Client->new(\%args);
+
+Valid constructor args are:
+
+* C<client_id> - Client identifier used for authorization, refresh token requests and unauthenticated requests.
+* C<client_secret> - Client secret used for acquiring a refresh token.
+* C<access_key> - Access token used to authenticate requests.
+* C<rapidapi_key> - Commercial use api key provided by RapidAPI.
+* C<format_type> - Api endpoint response format type. Options are C<json> (default) and C<xml>.
+* C<oauth_cb_state> - A parameter that's appended to the OAuth2 authorization callback URL. May be useful if you want to pass along a tracking value to the callback endpoint / collector.
+
+A getter and setter method is provided for each constructor arg.
 
 =head1 AUTHOR
 
@@ -860,14 +900,6 @@ Dillan Hildebrand
 
 MIT
 
-=head1 INSTALLATION
-
-Manual install:
-
-    $ perl Makefile.PL
-    $ make
-    $ make install
-
 =cut
 
 1;
